import asyncio
import logging
import collections
from functools import partial
import struct
import socket

from .constants import BUFSIZE
from .utils import detect_af


class SocksException(Exception):
    pass


class BadVersion(SocksException):
    pass


class BadAuthMethod(SocksException):
    pass


class BadAddress(SocksException):
    pass


SOCKS5REQ = struct.Struct('!BBBB')


class SocksListener:  # pylint: disable=too-many-instance-attributes
    def __init__(self, *,
                 listen_address,
                 listen_port,
                 pool,
                 timeout=4,
                 loop=None):
        self._loop = loop if loop is not None else asyncio.get_event_loop()
        self._logger = logging.getLogger(self.__class__.__name__)
        self._listen_address = listen_address
        self._listen_port = listen_port
        self._children = set()
        self._server = None
        self._pool = pool
        self._timeout = timeout

    async def stop(self):
        self._server.close()
        await self._server.wait_closed()
        while self._children:
            children = list(self._children)
            self._children.clear()
            self._logger.debug("Cancelling %d client handlers...",
                               len(children))
            for task in children:
                task.cancel()
            await asyncio.wait(children)
            # workaround for TCP server keeps spawning handlers for a while
            # after wait_closed() completed
            await asyncio.sleep(.5)

    async def _socks_prologue(self, reader, writer):
        ver = await reader.readexactly(1)
        if ver != b'\x05':
            raise BadVersion("Incorrect protocol version")

        n_methods = await reader.readexactly(1)
        n_methods = int.from_bytes(n_methods, 'big')
        if n_methods == 0:
            writer.write(b'\x05\xff')
            raise BadAuthMethod("Client didn't proposed any auth method, "
                                "even \"NO AUTHENTICATION REQUIRED\" method")

        methods = await reader.readexactly(n_methods)
        if b'\x00' not in methods:
            writer.write(b'\x05\xff')
            raise BadAuthMethod("Client didn't proposed the only suitable "
                                "\"NO AUTHENTICATION REQUIRED\" method")

        writer.write(b'\x05\x00')

        req_header = await reader.readexactly(SOCKS5REQ.size)
        req_ver, req_cmd, req_rsv, req_atyp = SOCKS5REQ.unpack(req_header)
        if req_ver != 5:
            ...
            raise BadVersion("Client specified inappropriate version "
                             "in connection request")
        if not (1 <= req_cmd <= 3):
            writer.write(b'\x05\x07')
            raise UnsupportedCommand("Client requested unsupported command")

        if req_atyp not in (1,3,4):
            writer.write(b'\x05\x08')
            raise UnsupportedAddress("Client requested connection to "
                                     "unsupported address type")

        if req_atyp == 3:
            # FQDN address
            fqdn_len = await reader.readexactly(1)
            fqdn_len = int.from_bytes(fqdn_len, 'big')
            if fqdn_len == 0:
                writer.write(b'\x05\x01')
                raise BadAddress("Client requested connection to 0-length "
                                 "domain name")
            address = (await reader.readexactly(fqdn_len)).decode('ascii')
        elif req_atyp == 1:
            # IPv4 address
            address = await reader.readexactly(4)
            address = socket.inet_ntoa(address)
        elif req_atyp == 4:
            # IPv6 address
            address = await reader.readexactly(16)
            address = socket.inet_ntop(socket.AF_INET6, address)
        port = await reader.readexactly(2)
        port = int.from_bytes(port, 'big')
        return req_cmd, address, port

    async def _socks_ok(self, reader, writer, peer):
        peer_addr, peer_port = peer
        peer_af = None
        try:
            peer_af = detect_af(peer_addr)
        except:
            pass
        if peer_af == socket.AF_INET:
            resp = (b'\x05\x00\x00\x01' + socket.inet_aton(peer_addr) +
                    peer_port.to_bytes(2, 'big'))
        elif peer_af == socket.AF_INET6:
            resp = (b'\x05\x00\x00\x04' + socket.inet_pton(AF_INET6, peer_addr) +
                    peer_port.to_bytes(2, 'big'))
        else:
            resp = b'\x05\x00\x00\x03\x00\x00\x00'
        self._logger.debug("Sending response to client: %s", resp.hex())
        writer.write(resp)

    async def _pump(self, writer, reader):
        while True:
            try:
                data = await reader.read(BUFSIZE)
            except asyncio.CancelledError:
                raise
            except ConnectionResetError:
                break
            if not data:
                break
            writer.write(data)

            try:
                await writer.drain()
            except ConnectionResetError:
                break
            except asyncio.CancelledError:
                raise

    async def handler(self, reader, writer):
        peer_addr = writer.transport.get_extra_info('peername')
        self._logger.info("Client %s connected", str(peer_addr))
        dst_writer = None
        try:
            cmd, dst_addr, dst_port = await self._socks_prologue(reader, writer)
            if cmd != 1:
                writer.write(b'\x05\x07')
                return
            self._logger.info("Client %s requested connection to %s:%s",
                              peer_addr, dst_addr, dst_port)
            async with self._pool.borrow() as ssh_conn:
                dst_reader, dst_writer = await asyncio.wait_for(
                    ssh_conn.open_connection(dst_addr, dst_port),
                    self._timeout)
                await self._socks_ok(reader, writer, writer.get_extra_info('sockname'))
                await asyncio.gather(self._pump(writer, dst_reader),
                                     self._pump(dst_writer, reader))
        except asyncio.CancelledError:  # pylint: disable=try-except-raise
            raise
        except Exception as exc:  # pragma: no cover
            self._logger.exception("Connection handler stopped with exception:"
                                   " %s", str(exc))
        finally:
            self._logger.info("Client %s disconnected", str(peer_addr))
            if dst_writer is not None:
                dst_writer.close()
            writer.close()

    async def start(self):
        def _spawn(reader, writer):
            def task_cb(task, fut):
                self._children.discard(task)
            task = self._loop.create_task(self.handler(reader, writer))
            self._children.add(task)
            task.add_done_callback(partial(task_cb, task))

        self._server = await asyncio.start_server(_spawn,
                                                  self._listen_address,
                                                  self._listen_port)
        self._logger.info("SOCKS5 server listening on %s:%d",
                          self._listen_address, self._listen_port)

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.stop()



import asyncio
from collections import deque

class Ratelimit:
    def __init__(self, rate, loop=None):
        self._loop = asyncio.get_event_loop() if loop is None else loop
        self._delay = 1. / rate
        self._last_released = 0
        self._waiters = deque()
        self._release_scheduled = False

    def _schedule_dispatch(self):
        self._loop.call_at(self._last_released + self._delay, self._dispatch)
        self._release_scheduled = True

    def _dispatch(self):
        self._release_scheduled = False
        while self._waiters:
            fut = self._waiters.popleft()
            if not fut.cancelled():
                fut.set_result(None)
                self._last_released = self._loop.time()
                break
        if self._waiters:
            self._schedule_dispatch()

    async def wait(self):
        t = self._loop.time()
        if self._last_released + self._delay < t and not self._release_scheduled:
            self._last_released = t
        else:
            fut = self._loop.create_future()
            self._waiters.append(fut)
            if not self._release_scheduled:
                self._schedule_dispatch()
            await fut

    async def __aenter__(self):
        await self.wait()
        return self

    async def __aexit__(self, exc, exc_type, tb):
        pass



import asyncio
import argparse
import logging
import logging.handlers
import ssl
import os
import queue
import socket
import ctypes

from . import constants


class OverflowingQueue(queue.Queue):
    def put(self, item, block=True, timeout=None):
        try:
            return queue.Queue.put(self, item, block, timeout)
        except queue.Full:
            pass

    def put_nowait(self, item):
        return self.put(item, False)


class AsyncLoggingHandler:
    def __init__(self, logfile=None, maxsize=1024):
        _queue = OverflowingQueue(maxsize)
        if logfile is None:
            _handler = logging.StreamHandler()
        else:
            _handler = logging.FileHandler(logfile)
        self._listener = logging.handlers.QueueListener(_queue, _handler)
        self._async_handler = logging.handlers.QueueHandler(_queue)

        _handler.setFormatter(logging.Formatter('%(asctime)s '
                                                '%(levelname)-8s '
                                                '%(name)s: %(message)s',
                                                '%Y-%m-%d %H:%M:%S'))

    def __enter__(self):
        self._listener.start()
        return self._async_handler

    def __exit__(self, exc_type, exc_value, traceback):
        self._listener.stop()


def setup_logger(name, verbosity, handler):
    logger = logging.getLogger(name)
    logger.setLevel(verbosity)
    logger.addHandler(handler)
    return logger


def check_port(value):
    def fail():
        raise argparse.ArgumentTypeError(
            "%s is not a valid port number" % value)
    try:
        ivalue = int(value)
    except ValueError:
        fail()
    if not 0 < ivalue < 65536:
        fail()
    return ivalue


def check_positive_float(value):
    def fail():
        raise argparse.ArgumentTypeError(
            "%s is not a valid value" % value)
    try:
        fvalue = float(value)
    except ValueError:
        fail()
    if fvalue <= 0:
        fail()
    return fvalue


def check_nonnegative_float(value):
    def fail():
        raise argparse.ArgumentTypeError(
            "%s is not a valid value" % value)
    try:
        fvalue = float(value)
    except ValueError:
        fail()
    if fvalue < 0:
        fail()
    return fvalue


def check_positive_int(value):
    def fail():
        raise argparse.ArgumentTypeError(
            "%s is not a valid value" % value)
    try:
        fvalue = int(value)
    except ValueError:
        fail()
    if fvalue <= 0:
        fail()
    return fvalue


def check_loglevel(arg):
    try:
        return constants.LogLevel[arg]
    except (IndexError, KeyError):
        raise argparse.ArgumentTypeError("%s is not valid loglevel" % (repr(arg),))


def enable_uvloop():  # pragma: no cover
    try:
        import uvloop
        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
    except ImportError:
        return False
    else:
        return True


def exit_handler(exit_event, signum, frame):  # pragma: no cover pylint: disable=unused-argument
    logger = logging.getLogger('MAIN')
    if exit_event.is_set():
        logger.warning("Got second exit signal! Terminating hard.")
        os._exit(1)  # pylint: disable=protected-access
    else:
        logger.warning("Got first exit signal! Terminating gracefully.")
        exit_event.set()


async def heartbeat():
    """ Hacky coroutine which keeps event loop spinning with some interval
    even if no events are coming. This is required to handle Futures and
    Events state change when no events are occuring."""
    while True:
        await asyncio.sleep(.5)


def detect_af(addr):
    return socket.getaddrinfo(addr,
                              None,
                              socket.AF_UNSPEC,
                              0,
                              0,
                              socket.AI_NUMERICHOST)[0][0]



#!/usr/bin/env python3

import sys
import argparse
import asyncio
import os
import os.path

import asyncssh

from . import utils


def parse_args():
    parser = argparse.ArgumentParser(
        description="Rapid SSH Proxy: TOFU key trust utility",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("dst_address",
                        help="target hostname")
    parser.add_argument("dst_port",
                        nargs="?",
                        default=22,
                        type=utils.check_port,
                        help="target port")

    ssh_group = parser.add_argument_group('SSH options')
    ssh_group.add_argument("-H", "--hosts-file",
                           default=os.path.join(os.path.expanduser("~"),
                                                '.rsp',
                                                'known_hosts'),
                           help="overrides known_hosts file location",
                           metavar="FILE")

    return parser.parse_args()


def main():  # pragma: no cover
    args = parse_args()
    if os.access(args.hosts_file, os.R_OK):
        known_hosts = asyncssh.read_known_hosts(args.hosts_file)
        match = known_hosts.match(args.dst_address, "0.0.0.0", args.dst_port)[0]
        if match:
            print("Host already added to known_hosts", file=sys.stderr)
            exit(4)
    loop = asyncio.get_event_loop()
    hostkey = loop.run_until_complete(
        asyncssh.get_server_host_key(args.dst_address, args.dst_port))
    loop.close()
    if hostkey is None:
        print("Unable to retrieve hostkey", file=sys.stderr)
        exit(3)
    print("%s key fingerprint is %s." % (hostkey.get_algorithm(),
                                         hostkey.get_fingerprint()))
    inp = input("Do you want to trust this key (yes/no)? ").lower()
    while True:
        if inp == 'yes':
            hostkey_export = hostkey.export_public_key('openssh').\
                decode('ascii').\
                rstrip('\n')
            os.makedirs(os.path.dirname(args.hosts_file), mode=0o700, exist_ok=True)
            with open(args.hosts_file, 'a') as f:
                print("%s %s" % (args.dst_address, hostkey_export), file=f)
                exit(0)
        elif inp == 'no':
            exit(0)
        else:
            inp = input("Please type 'yes' or 'no': ").lower()
        

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import sys
import argparse
import asyncio
import os
import os.path

import asyncssh

KEY_TYPES = [
    "ssh-ed25519",
    "ssh-rsa",
    "ecdsa-sha2-nistp256",
    "ecdsa-sha2-nistp384",
    "ecdsa-sha2-nistp521",
    "ssh-dss",
]


def check_keysize(value):
    def fail():
        raise argparse.ArgumentTypeError(
            "%s is not a valid RSA key size" % value)
    try:
        ivalue = int(value)
    except ValueError:
        fail()
    if not (2048 <= ivalue <= 8192):
        fail()
    return ivalue


def parse_args():
    parser = argparse.ArgumentParser(
        description="Rapid SSH Proxy: key generation utility",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("-f", "--file",
                        default="proxy_key",
                        help="output file name")
    parser.add_argument("-t", "--type",
                        default=KEY_TYPES[0],
                        choices=KEY_TYPES,
                        help="key type")
    parser.add_argument("-b", "--bits",
                        default=2048,
                        type=check_keysize,
                        help="key type")

    return parser.parse_args()


def main():  # pragma: no cover
    args = parse_args()
    opts = {}
    if args.type == "ssh-rsa":
        opts['key_size'] = args.bits

    privkey = asyncssh.generate_private_key(args.type, **opts)
    try:
        with open(args.file, 'xb') as sk_file:
            sk_export = privkey.export_private_key(format_name='openssh')
            sk_file.write(sk_export)
        print('Your identification has been saved in %s.' % (args.file,))
        pub_filename = args.file + '.pub'
        with open(pub_filename, 'xb') as pk_file:
            pk_export = privkey.export_public_key(format_name='openssh')
            pk_file.write(pk_export)
        print('Your public key has been saved in %s.' % (pub_filename,))
    except FileExistsError as exc:
        print("Error: file '%s' already exists." % (str(exc.filename),), file=sys.stderr)


if __name__ == '__main__':
    main()



import asyncio
import logging
import collections
from functools import partial

import asyncssh


class SSHPoolBorrow:
    def __init__(self, get, release):
        self._conn = None
        self._get = get
        self._release = release

    async def __aenter__(self):
        self._conn = await self._get()
        return self._conn

    async def __aexit__(self, exc, exc_type, tb):
        if exc_type is None:
            self._release(self._conn)


class SSHPool:
    def __init__(self, *,
                 dst_address,
                 dst_port,
                 ratelimit,
                 ssh_options=None,
                 timeout=4,
                 backoff=5,
                 size=15,
                 loop=None):
        self._logger = logging.getLogger(self.__class__.__name__)
        self._loop = loop if loop is not None else asyncio.get_event_loop()
        self._dst_address = dst_address
        self._dst_port = dst_port
        self._timeout = timeout
        self._ssh_options = ssh_options
        self._size = size
        self._backoff = backoff
        self._waiters = collections.deque()
        self._reserve = collections.deque()
        self._ratelimit = ratelimit
        self._tasks = set()

    async def start(self):
        self._rebalance_pool()

    async def stop(self):
        while self._tasks:
            tasks = list(self._tasks)
            self._tasks.clear()
            for t in tasks:
                t.cancel()
            await asyncio.gather(*tasks, return_exceptions=True)
        for conn in self._reserve:
            conn.abort()

    def _task_done_cb(self, task):
        if not task.cancelled():
            exc = task.exception()
            if exc is not None:
                try:
                    raise exc
                except Exception:
                    self._logger.exception("Got exception from finished pool "
                                           "task: %s", str(exc))
        self._tasks.discard(task)

    def _rebalance_pool(self):
        debt = self._size - len(self._reserve) + len(self._waiters) - len(self._tasks)
        self._logger.debug("_rebalance_pool: debt=%d; len(reserve)=%d, "
                           "len(waiters)=%d, len(tasks)=%d", debt,
                           len(self._reserve), len(self._waiters),
                           len(self._tasks))
        for i in range(debt):
            task = self._loop.create_task(self._build_conn())
            task.add_done_callback(self._task_done_cb)
            self._tasks.add(task)

    async def _build_conn(self):
        async def fail():
            self._logger.debug("Failed upstream connection. Backoff for %d "
                               "seconds", self._backoff)
            await asyncio.sleep(self._backoff)

        while True:
            try:
                async with self._ratelimit:
                    self._logger.debug("_build_conn: connect attempt.")
                    conn = await asyncio.wait_for(
                        asyncssh.connect(self._dst_address,
                                         self._dst_port,
                                         options=self._ssh_options()),
                        self._timeout)
                    break
            except asyncio.TimeoutError:
                self._logger.error("Connection to upstream timed out.")
                await fail()
            except asyncio.CancelledError:
                raise
            except Exception as exc:
                self._logger.exception("Got exception while connecting to upstream: %s", str(exc))
                await fail()
        self._logger.debug("Successfully built upstream connection.")
        while self._waiters:
            fut = self._waiters.popleft()
            if not fut.cancelled():
                self._logger.warning("Pool exhausted. Dispatching connection directly to waiter!")
                fut.set_result(conn)
                break
        else:
            self._reserve.append(conn)
                    
    async def get(self):
        if self._reserve:
            conn = self._reserve.popleft()
            self._rebalance_pool()
            self._logger.debug("Obtained connection from pool.")
            return conn
        else:
            fut = self._loop.create_future()
            self._waiters.append(fut)
            self._rebalance_pool()
            self._logger.debug("Awaiting for free connection.")
            return await fut

    def release(self, conn):
        self._logger.debug("Connection released.")
        self._reserve.append(conn)

    def borrow(self):
        return SSHPoolBorrow(self.get, self.release)

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.stop()



#!/usr/bin/env python3

import sys
import argparse
import asyncio
import logging
import signal
from functools import partial
import os.path

from sdnotify import SystemdNotifier
import asyncssh

from .listener import SocksListener
from .constants import LogLevel
from . import utils
from .ssh_pool import SSHPool
from .ratelimit import Ratelimit


def parse_args():
    parser = argparse.ArgumentParser(
        description="Rapid SSH Proxy",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("dst_address",
                        help="target hostname")
    parser.add_argument("dst_port",
                        nargs="?",
                        default=22,
                        type=utils.check_port,
                        help="target port")
    parser.add_argument("-v", "--verbosity",
                        help="logging verbosity",
                        type=utils.check_loglevel,
                        choices=LogLevel,
                        default=LogLevel.info)
    parser.add_argument("-l", "--logfile",
                        help="log file location",
                        metavar="FILE")
    parser.add_argument("--disable-uvloop",
                        help="do not use uvloop even if it is available",
                        action="store_true")

    listen_group = parser.add_argument_group('listen options')
    listen_group.add_argument("-a", "--bind-address",
                              default="127.0.0.1",
                              help="bind address")
    listen_group.add_argument("-p", "--bind-port",
                              default=1080,
                              type=utils.check_port,
                              help="bind port")

    pool_group = parser.add_argument_group('pool options')
    pool_group.add_argument("-n", "--pool-size",
                            default=30,
                            type=utils.check_positive_int,
                            help="target number of steady connections")
    pool_group.add_argument("-B", "--backoff",
                            default=5,
                            type=utils.check_positive_float,
                            help="delay after connection attempt failure in seconds")
    pool_group.add_argument("-w", "--timeout",
                            default=4,
                            type=utils.check_positive_float,
                            help="server connect timeout")
    pool_group.add_argument("-r", "--connect-rate",
                            default=0.5,
                            type=utils.check_nonnegative_float,
                            help="limit for new pool connections per second")

    ssh_group = parser.add_argument_group('SSH options')
    ssh_group.add_argument("-L", "--login",
                           help="SSH login. Default is name of current user")
    ssh_group.add_argument("-I", "--identity",
                           action="append",
                           help="SSH private key file. By default program looks "
                           "for SSH keys in usual locations, including SSH "
                           "agent socket. This option may be specified "
                           "multiple times",
                           metavar="KEY_FILE")
    ssh_group.add_argument("-P", "--password",
                           help="SSH password. If not specified, password auth"
                           " will be disabled")
    ssh_group.add_argument("-H", "--hosts-file",
                           default=os.path.join(os.path.expanduser("~"),
                                                '.rsp',
                                                'known_hosts'),
                           help="overrides known_hosts file location",
                           metavar="FILE")

    return parser.parse_args()

def ssh_options_from_args(args, known_hosts):
    kw = dict()
    kw['gss_host'] = None
    kw['known_hosts'] = known_hosts
    if args.login is not None:
        kw['username'] = args.login
    if args.identity is not None:
        kw['client_keys'] = list(args.identity)
    if args.password is not None:
        kw['password'] = args.password
    return asyncssh.SSHClientConnectionOptions(**kw)


async def amain(args, loop):  # pragma: no cover
    logger = logging.getLogger('MAIN')
    known_hosts = asyncssh.read_known_hosts(args.hosts_file)
    matched_keys = known_hosts.match(args.dst_address, "", args.dst_port)[0]
    if not matched_keys:
        logger.critical("Specified host is not found in known hosts. "
                        "Please run following command: "
                        "rsp-trust '%s' %d",
                        args.dst_address, args.dst_port)
        return
    options = partial(ssh_options_from_args, args, known_hosts)

    ratelimit = Ratelimit(args.connect_rate)
    pool = SSHPool(dst_address=args.dst_address,
                   dst_port=args.dst_port,
                   ssh_options=options,
                   timeout=args.timeout,
                   backoff=args.backoff,
                   ratelimit=ratelimit,
                   size=args.pool_size,
                   loop=loop)
    async with pool:
        logger.warning("SSH connection pool is starting up. Pool target: "
                       "%d steady connections. It will take at least %.2f "
                       "seconds to reach it's full size.", args.pool_size,
                       args.pool_size * 1. / args.connect_rate)
        server = SocksListener(listen_address=args.bind_address,
                          listen_port=args.bind_port,
                          timeout=args.timeout,
                          pool=pool,
                          loop=loop)
        async with server:
            logger.info("Server started.")

            exit_event = asyncio.Event()
            beat = asyncio.ensure_future(utils.heartbeat())
            sig_handler = partial(utils.exit_handler, exit_event)
            signal.signal(signal.SIGTERM, sig_handler)
            signal.signal(signal.SIGINT, sig_handler)
            notifier = await loop.run_in_executor(None, SystemdNotifier)
            await loop.run_in_executor(None, notifier.notify, "READY=1")
            await exit_event.wait()

            logger.debug("Eventloop interrupted. Shutting down server...")
            await loop.run_in_executor(None, notifier.notify, "STOPPING=1")
            beat.cancel()


def main():  # pragma: no cover
    args = parse_args()
    with utils.AsyncLoggingHandler(args.logfile) as log_handler:
        logger = utils.setup_logger('MAIN', args.verbosity, log_handler)
        utils.setup_logger('SocksListener', args.verbosity, log_handler)
        utils.setup_logger('SSHPool', args.verbosity, log_handler)

        logger.info("Starting eventloop...")
        if not args.disable_uvloop:
            if utils.enable_uvloop():
                logger.info("uvloop enabled.")
            else:
                logger.info("uvloop is not available. "
                            "Falling back to built-in event loop.")

        loop = asyncio.get_event_loop()
        loop.run_until_complete(amain(args, loop))
        loop.close()
        logger.info("Server finished its work.")


if __name__ == '__main__':
    main()



import enum
import logging


class LogLevel(enum.IntEnum):
    debug = logging.DEBUG
    info = logging.INFO
    warn = logging.WARN
    error = logging.ERROR
    fatal = logging.FATAL
    crit = logging.CRITICAL

    def __str__(self):
        return self.name


BUFSIZE = 16 * 1024



from setuptools import setup

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='rsp',
      version='0.2.1',
      description='Rapid SSH Proxy',
      url='https://github.com/Snawoot/rsp',
      author='Vladislav Yarmak',
      author_email='vladislav-ex-src@vm-0.com',
      license='MIT',
      packages=['rsp'],
      python_requires='>=3.5.3',
      setup_requires=[
          'wheel',
      ],
      install_requires=[
          'sdnotify>=0.3.2',
          'asyncssh>=1.16.0',
      ],
      extras_require={
          'dev': [
              'setuptools>=38.6.0',
              'wheel>=0.31.0',
              'twine>=1.11.0',
          ],
      },
      entry_points={
          'console_scripts': [
              'rsp=rsp.__main__:main',
              'rsp-trust=rsp.trust:main',
              'rsp-keygen=rsp.keygen:main',
          ],
      },
      classifiers=[
          "Programming Language :: Python :: 3.5",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Development Status :: 4 - Beta",
          "Environment :: No Input/Output (Daemon)",
          "Intended Audience :: System Administrators",
          "Natural Language :: English",
          "Topic :: Internet",
          "Topic :: Security",
      ],
      long_description=long_description,
      long_description_content_type='text/markdown',
      zip_safe=True)
